//---------------------------------------------------------------------
//  Algorithmic Conjurings @ http://www.coyotegulch.com
//  Evocosm -- An Object-Oriented Framework for Evolutionary Algorithms
//
//  reproducer.h
//---------------------------------------------------------------------
//
//  COPYRIGHT NOTICE, DISCLAIMER, and LICENSE:
//
//  This notice applies *only* to this specific expression of this
//  algorithm, and does not imply ownership or invention of the
//  implemented algorithm.
//  
//  If you modify this file, you may insert additional notices
//  immediately following this sentence.
//  
//  Copyright 2002, 2003 Scott Robert Ladd. All rights reserved.
//  All rights reserved, except as noted herein.
//
//  This computer program source file is supplied "AS IS". Scott Robert
//  Ladd (hereinafter referred to as "Author") disclaims all warranties,
//  expressed or implied, including, without limitation, the warranties
//  of merchantability and of fitness for any purpose. The Author
//  assumes no liability for direct, indirect, incidental, special,
//  exemplary, or consequential damages, which may result from the use
//  of this software, even if advised of the possibility of such damage.
//  
//  The Author hereby grants anyone permission to use, copy, modify, and
//  distribute this source code, or portions hereof, for any purpose,
//  without fee, subject to the following restrictions:
//  
//      1. The origin of this source code must not be misrepresented.
//  
//      2. Altered versions must be plainly marked as such and must not
//         be misrepresented as being the original source.
//  
//      3. This Copyright notice may not be removed or altered from any
//         source or altered source distribution.
//  
//  The Author specifically permits (without fee) and encourages the use
//  of this source code for entertainment, education, or decoration. If
//  you use this source code in a product, acknowledgment is not required
//  but would be appreciated.
//  
//  Acknowledgement:
//      This license is based on the wonderful simple license that
//      accompanies libpng.
//
//-----------------------------------------------------------------------
//  For more information on this software package, please visit
//  Scott's web site, Coyote Gulch Productions, at:
//
//      http://www.coyotegulch.com
//-----------------------------------------------------------------------

#if !defined(LIBEVOCOSM_REPRODUCER_H)
#define LIBEVOCOSM_REPRODUCER_H

// libevocosm
#include "libevocosm/organism.h"

namespace libevocosm
{
    //! Creates new organisms from an existing population
    /*!
        In most cases, a reproducer generates new organisms using parents
        selected (by fitness) from an existing population. In some singular
        (and probably rare) cases, a reproducer might generate new, random
        organisms in order to keep diversity high.
        Reproduction techniques can include crossover and asexual, sexual
        and (my favorite) try-sexual models.
        \param OrganismType - The type of organism
    */
    template <class OrganismType>
    class reproducer : protected globals
    {
    public:
        //! Virtual destructor
        /*!
            A virtual destructor. By default, it does nothing; this is
            a placeholder that identifies this class as a potential base,
            ensuring that objects of a derived class will have their
            destructors called if they are destroyed through a base-class
            pointer.
        */
        virtual ~reproducer()
        {
            // nada
        }

        //! Creates children
        /*!
            A population provides genetic material for the creation of new children (who,
            will, like real children, forget where they left their shoes).
            \param a_population - A population of organisms
            \param a_limit - The maximum number of children the generate
            \return A vector containing new "child" organisms
        */
        virtual vector<OrganismType> breed(const vector<OrganismType> & a_population, size_t a_limit) = 0;
    };
};

#endif
